#ifdef __CINT__

#pragma link off all globals;
#pragma link off all classes;
#pragma link off all functions;

#pragma link C++ global FR.*;
#pragma link C++ enum FR.*;
#pragma link C++ class Fr.*;
#pragma link C++ function Fr.*;
#pragma link off function FrEventNew;
#pragma link off function FrEventReadTF;
#pragma link off function FrSimEventReadTF;

#endif
